package com.iden.common.interceptor;

import cn.hutool.core.util.StrUtil;
import com.iden.common.annotation.LogModule;
import com.iden.common.annotation.LoginedUser;
import com.iden.common.annotation.Permission;
import com.iden.common.constant.Constants;
import com.iden.common.util.PageResult;
import com.iden.common.util.Result;
import com.iden.common.util.ResultCode;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.UserLoginedConvertVO;
import io.swagger.annotations.ApiOperation;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

/**
 * Created by y on 2017/11/16.
 * @author y
 */
@Component
@Aspect
public class ControllerInterceptor {

    private static final Logger logger = LoggerFactory.getLogger(ControllerInterceptor.class);


//    @Autowired
//    SysLogService sysLogService;

    /**
     * 定义拦截规则：拦截com.bj106.zhyc.app.controller包下面的所有类中，有@RequestMapping注解的方法。
     */
    @Pointcut("execution(public * com.face..*.controller..*(..))")
    public void controllerMethodPointcut(){}

    /**
     * 拦截器具体实现<br/>
     * 指定拦截器规则；也可以直接把“execution(* com.bj106.zhyc.........)”写进这里
     * @param pjp
     * @return JsonResult（被拦截方法的执行结果，或需要登录的错误提示。）
     */
    @Around("controllerMethodPointcut()")
    public Object interceptor(ProceedingJoinPoint pjp){

        long beginTime = System.currentTimeMillis();
        MethodSignature signature = (MethodSignature)pjp.getSignature();

//        SysLogDO sysLogDO = new SysLogDO();

        /**
         * 获取被拦截的方法
         */
        Method method = signature.getMethod();
        /**
         * 获取被拦截的方法名
         */
        String methodName = method.getName();
//        sysLogDO.setMethod(methodName);
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

//        sysLogDO.setUrl(request.getRequestURI().toString());

        /**
         * 保存所有请求参数，用于输出到日志中
         */
        Set<Object> allParams = new LinkedHashSet<>();
        Object result = null;
        logger.debug("请求开始，方法：{}", (pjp.getTarget().getClass().getName() + "." + pjp.getSignature().getName() + "()"));
        try {
            String description = getControllerMethodDescription(pjp);
            if (StrUtil.isNotBlank(description)) {
                logger.debug("方法描述: " + description);
//                sysLogDO.setDescription(description);
            }

            String[] moduleNames = getControllerModule(pjp);
            if (moduleNames.length > 0) {
//                sysLogDO.setModuleName(moduleNames[0]);
                if (moduleNames.length == 2) {
//                    sysLogDO.setChildModuleName(moduleNames[1]);
                }
            }

            String ip = WebPageUtils.getIpAddr(request);
            logger.debug("请求IP: " + ip);
//            sysLogDO.setIp(ip);
            UserLoginedConvertVO dto = (UserLoginedConvertVO)request.getAttribute(Constants.CURRENT_LOGINED_USER_KEY);
            if (dto != null) {
//                UserDO userDO = new UserDO();
//                userDO.setId(dto.getId());
//                sysLogDO.setOperUserDO(userDO);
//                logger.debug("请求人: " + dto.getRealName());
            } else {
                if (isLoginRequired(pjp.getTarget().getClass(), method)) {
                    logger.debug("请求异常: 该接口需要登录后进行操作. 请先登录", methodName, System.currentTimeMillis() - beginTime);
                    logger.debug("请求结束，方法：{} -> 耗时：{}ms", methodName, System.currentTimeMillis() - beginTime);
                    return new PageResult(ResultCode.NO_LOGINED, "该操作需要登录.",null);
                }
            }
            /**
             * 请求参数
             */
            Object[] args = pjp.getArgs();
            for (Object arg : args) {
                if (arg instanceof Map<?, ?>) {
                    /**
                     * 提取方法中的MAP参数，用于记录进日志中
                     */
                    @SuppressWarnings("unchecked")
                    Map<String, Object> map = (Map<String, Object>)arg;
                    allParams.add(map);
                } else if (arg instanceof HttpServletRequest) {
                    HttpServletRequest argRequest = (HttpServletRequest)arg;
                    //获取query string 或 posted form data参数
                    Map<String, String[]> paramMap = argRequest.getParameterMap();
                    if (paramMap!=null && paramMap.size() > 0) {
                        allParams.add(paramMap);
                    }
                } else if (arg instanceof HttpServletResponse) {
                    //do nothing...
                } else {
                    allParams.add(arg);
                }
            }

            if (!"true".equalsIgnoreCase(request.getParameter("interval"))) {
//                sysLogService.saveOrUpdate(sysLogDO);
            }
        } catch (Exception e) {
            logger.error("异常信息:{}", e.getMessage());
        }
        try {
            if(result == null) {
                /**
                 * 一切正常的情况下，继续执行被拦截的方法
                 */
                result = pjp.proceed();
            }
        } catch (Throwable e) {
            logger.error("exception: ", e);
            result = new PageResult(ResultCode.UNKNOWN, StrUtil.isNotBlank(e.getMessage()) ? e.getMessage() : "请求服务器出现异常, 请稍后重试.",e.getMessage());
        }
        if (result instanceof Result) {
            logger.debug("请求结束，方法：{} -> 耗时：{}ms", methodName, System.currentTimeMillis() - beginTime);
        }
        return result;
    }


    /**
     * 判断一个方法是否需要登录
     * @param method
     * @return
     */
    private boolean isLoginRequired(Class<?> clazz, Method method){
//        if(!env.equals("prod")){ //只有生产环境才需要登录
//            return false;
//        }

        boolean result = false;

        /**
         * 首先校验方法是否存在 登录 注解, 并根据此来进行是否需要验证拦截
         */
        if(method.isAnnotationPresent(Permission.class)){
            result = method.getAnnotation(Permission.class).loginReqired();
        }
        /**
         * 如方法无注解, 则根据类是否添加 登录 注解, 并根据此来进行是否需要验证拦截
         */
        else if (clazz.isAnnotationPresent(Permission.class)) {
            result = clazz.getAnnotation(Permission.class).loginReqired();
        } else {
            Annotation[][] annotations = method.getParameterAnnotations();
            for (Annotation[] childAnnotations : annotations) {
                for (Annotation childAnnotation : childAnnotations) {
                    /**
                     * 如果方法参数中存在 LoginedUser 属性获取注解, 则也认定为需要 登录 验证
                     */
                    if (childAnnotation.annotationType() == LoginedUser.class) {
                        return true;
                    }
                }
            }
        }
        return result;
    }

    /**
     * 获取注解中对类的描述信息 用于Controller层注解
     *
     * @param joinPoint 切点
     * @return 类描述
     * @throws Exception
     */
//    public  static String getControllerDescription(JoinPoint joinPoint)  throws Exception {
//        String targetName = joinPoint.getTarget().getClass().getName();
//        String methodName = joinPoint.getSignature().getName();
//        Object[] arguments = joinPoint.getArgs();
//        Class targetClass = Class.forName(targetName);
//        Method[] methods = targetClass.getMethods();
//        String description = "";
//        for (Method method : methods) {
//            if (method.getName().equals(methodName)) {
//                Class[] clazzs = method.getParameterTypes();
//                if (clazzs.length == arguments.length) {
//                    Api api = (Api)targetClass.getAnnotation(Api.class);
//                    if (api != null) {
//                        description = api.description();
//                    }
//                    break;
//                }
//            }
//        }
//        return description;
//    }

    /**
     * 获取注解中对方法的描述信息 用于Controller层 Method 注解
     *
     * @param joinPoint 切点
     * @return 方法描述
     * @throws Exception
     */
    public  static String getControllerMethodDescription(JoinPoint joinPoint)  throws Exception {
        String targetName = joinPoint.getTarget().getClass().getName();
        String methodName = joinPoint.getSignature().getName();
        Object[] arguments = joinPoint.getArgs();
        Class targetClass = Class.forName(targetName);
        Method[] methods = targetClass.getMethods();
        String description = "";
        for (Method method : methods) {
            if (method.getName().equals(methodName)) {
                Class[] clazzs = method.getParameterTypes();
                if (clazzs.length == arguments.length) {
                    ApiOperation apiOperation = method.getAnnotation(ApiOperation.class);
                    if (apiOperation != null) {
                        description = apiOperation.value();
                        if (StrUtil.isBlank(description)) {
                            description = apiOperation.notes();
                        }
                    }
                    break;
                }
            }
        }
        return description;
    }

    /**
     * 获取注解中对方法的描述信息 用于Controller层 Method 注解
     *
     * @param joinPoint 切点
     * @return 方法描述
     * @throws Exception
     */
    public static String[] getControllerModule(JoinPoint joinPoint)  throws Exception {
        String targetName = joinPoint.getTarget().getClass().getName();
        String methodName = joinPoint.getSignature().getName();
        Object[] arguments = joinPoint.getArgs();
        Class targetClass = Class.forName(targetName);
        Method[] methods = targetClass.getMethods();
        for (Method method : methods) {
            if (method.getName().equals(methodName)) {
                Class[] clazzs = method.getParameterTypes();
                if (clazzs.length == arguments.length) {
                    LogModule logModule = method.getAnnotation(LogModule.class);
                    if (logModule != null) {
                        if (StrUtil.isNotBlank(logModule.childModule())) {
                            return new String[] { logModule.masterModule(), logModule.childModule() };
                        }
                        return new String[] { logModule.masterModule() };
                    }
                    break;
                }
            }
        }
        LogModule logControllerAnnotation = AnnotationUtils.findAnnotation(targetClass, LogModule.class);
        if (logControllerAnnotation != null) {
            if (StrUtil.isNotBlank(logControllerAnnotation.childModule())) {
                return new String[] { logControllerAnnotation.masterModule(), logControllerAnnotation.childModule() };
            }
            return new String[] { logControllerAnnotation.masterModule() };
        }
        return new String[0];
    }

}
